# -*- coding: utf-8 -*-
#
# Copyright 2016 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os.path
from gettext import NullTranslations, translation


translation_dir = os.path.join(
    os.path.dirname(
        os.path.abspath(
            __file__,
        ),
    ),
    "translations"
)
current_translation = NullTranslations()


def set_locale(locales):
    global current_translation
    current_translation = translation(
        domain="rqalpha",
        localedir=translation_dir,
        languages=locales,
    )


def gettext(message):
    return current_translation.gettext(message)
